#include "mdc_cam.h"

using namespace mdc::camera;
using namespace mdc;

MDCCamera::MDCCamera(std::shared_ptr<CameraPool> pool)
{
    this->pool_ = pool;
}

void MDCCamera::init(std::vector<mdc::camera::CameraAccessConfig> &configs)
{
    MDCMbufSysInit();
    if (mdc::camera::CameraInit(configs))
    {
        this->logger.LogFatal() << "Camera init failed";
    }
    else
    {
        this->logger.LogDebug() << "Camera init completed";
        // auto temp = mdc::camera::CameraGetInternalData(8U);
        // for (const auto &t: temp)
        // {
        //     printf("%d ",t);
        // }
        // printf("\nsize: %d", temp.size());
        this->ready = true;
        this->act = std::thread(&MDCCamera::runCameras, this);
        this->logger.LogDebug() << "Camera read data thread created";
    }
}

std::vector<CameraImageResult> MDCCamera::getCameraImages()
{
    std::vector<uint32_t> camIDs;
    camIDs = CameraWaitEvents(-1);
    std::vector<CameraImageResult> result;
    if (!camIDs.empty())
    {
        for (auto id : camIDs)
        {
            mdc::camera::CameraData image;
            uint8_t failed = CameraGetData(id, image, image_timeout);
            if (!failed)
            {
                CameraImageResult img_res;
                img_res.id = id;
                img_res.image = image;
                result.push_back(img_res);
                this->logger.LogInfo() << "Camera " << id << "Image successfully pushed to the vector";
            }
            else
            {
                this->logger.LogError() << "Camera" << id << "failed to get data with error " << failed;
            }
        }
    }
    else
    {
        this->logger.LogError() << "No Camera Ready";
    }
    return result;
}

uint8_t MDCCamera::getRawId(uint8_t id)
{
    return id % this->camStartId;
}

uint8_t MDCCamera::getCameraImagesArray(std::array<CameraImageResult, 4> &result)
{
    std::vector<uint32_t> camIDs;
    camIDs = CameraWaitEvents(-1);
    if (!camIDs.empty())
    {
        this->logger.LogDebug() << "length cameraIDs: " << camIDs.size();
        for (auto id : camIDs)
        {
            uint8_t rawId = this->getRawId(id);
            uint8_t failed = CameraGetData(id, result[rawId].image, image_timeout);
            if (!failed)
            {
                result[rawId].id = id;
                result[rawId].updated = true;
                this->logger.LogInfo() << "Camera " << id << "Image successfully pushed to the vector";
            }
            else
            {
                this->logger.LogError() << "Camera" << id << "failed to get data with error " << failed;
            }
        }
    }
    else
    {
        this->logger.LogError() << "No Camera Ready";
    }
    return camIDs.size();
}

uint8_t MDCCamera::getCameraImagesArrayVector(std::array<CameraImageResult, 4> &result)
{
    std::vector<uint32_t> camIDs;
    this->logger.LogDebug() << "Attempt to getting camera IDs via CameraWaitEvents method";
    camIDs = CameraWaitEvents(-1);
    if (!camIDs.empty())
    {
        for (auto id : camIDs)
        {
            uint8_t rawId = this->getRawId(id);
            std::lock_guard<std::mutex> lock(this->pool_->mtx[rawId]);
            uint8_t failed = CameraGetData(id, result[rawId].imageVector, image_timeout);
            if (!failed)
            {
                result[rawId].id = id;
                this->pool_->dataReady[rawId] = true;
                this->pool_->cv[rawId].notify_one();
                this->logger.LogDebug() << "Camera " << id << " Image successfully pushed to the vector";
            }
            else
            {
                this->logger.LogError() << "Camera" << id << "failed to get data with error " << failed;
            }
        }
    }
    else
    {
        this->logger.LogError() << "No Camera Ready";
    }
    return camIDs.size();
}

uint8_t MDCCamera::getCameraYUVBuffer(void **buffer, size_t &length, mdc::camera::CameraData &image)
{
    uint8_t retValue = image.yuvData.GetBuffData(*buffer, length);
    retValue = retValue == 0 ? 1 : 0;
    return retValue;
}

void MDCCamera::runCameras()
{
    this->logger.LogDebug() << "Thread start running";
    uint8_t cameraSize;
    while (this->ready)
    {
        if (this->pool_ != nullptr)
        {
            cameraSize = this->getCameraImagesArrayVector(this->pool_->pool);
            this->logger.LogDebug() << "number of ready camera: " << cameraSize;
        }
        else
        {
            this->logger.LogError() << "No Camera Ready";
        }
    }
}